function[U,coretensor,tapprox,terr]=m_mode_svd_reduce(tensor,modes,ranks,eps,niter,U,coretensor);
% "tensor" is a data tensor
% "ranks" is a row vector that specifies the desired ranks for each mode
% "eps" maximum allowable error for each mode matrix
% "niter" maximum number of iterations
% "U"contains all the untruncated mode matrices computed by m_mode_svd
% "coretensor" computed by m_mode_svd
%
% Please report bugs/fixes to maov@mrl.nyu.edu
% Copyright 2005-2025 M.A.O.Vasilescu
% Please do NOT redistribute without permission; direct requests to maov@mrl.nyu.edu

% Paper ECCV'02, vol.2350, pg.447-460./Paper ICPR'02, vol.3 pg.456-460.
% Paper CVPR'03, vol.2, pg.93-99.
% Created 2001 and Revised Feb 28, 2005 by M.A.O.Vasilescu


dims=size(tensor)
if nargin==5
  [U, coretensor]=m_mode_svd(tensor,modes);
elseif nargin<5
 error('Not enough inputs.  Must provide at least 4 inputs.\n')
elseif nargin<7
 error('Not enough inputs. Must provide either 4 or 6 inputs.\n')
end




opt_cond=1;
iter=2;

while opt_cond & iter<niter
  for i=modes
    rmodes = setxor(modes,i);
    [U,old_U,mode_err] = optimize_mode(tensor,U,modes(i),rmodes,ranks(i),ranks);
    err(i)=mode_err;
  end

  coretensor=tensor;
  for i=modes
     coretensor= mode_m_prod(coretensor,U{i}',i);
  end

  tapprox=coretensor;
  for i=modes
     tapprox= mode_m_prod(tapprox,U{i},i);
  end

  fta=matricize(tapprox,5);
  terr(iter)=frobenius_norm(tensor-tapprox)^2
  
  iter=iter+1;
  opt_cond=iter_condition(err,eps,ranks,modes);
end

