function [Tn] = tensorize(matrix,n,dims)
% Retensorize "matrix" which was previously flattened in the N-th dimension.
% dims=[a b c d...] are the dimensions of the tensor Tn
%
% Issues: If the last dimension of your tensor is a singleton, it will not be 
%         represented explicitly. See the difference between Example 2 and 3.
%
% Example 1: 
%	   randn('seed',0)
%          M=randn(3,4)
%          tensor=retensorize(M,2,[2 3 2])
%          size(tensor)
%
% Example 2: 
%          tensor=retensorize(M,2,[1 3 4])
%          size(tensor)
%
% Example 3:
%          tensor=retensorize(M,2,[3 4 1])
%          size(tensor)
%
% Please report bugs/fixes to maov@mrl.nyu.edu
% Copyright 2005-2025 M.A.O.Vasilescu
% Please do NOT redistribute without permission; direct requests to maov@mrl.nyu.edu

% Paper ECCV'02, vol.2350, pg.447-460./Paper ICPR'02, vol.3 pg.456-460.
% Paper CVPR'03, vol.2, pg.93-99.
% Created 2001 and Revised Feb 28, 2005 by M.A.O.Vasilescu


ndims=wshift('1D',dims,n-1);
Tn = reshape(matrix,ndims);
l=length(dims);
Tn=shiftdim(Tn,l-n+1);
if length((ndims)) ~= length(size(Tn))
  if dims(length(dims))~=1
   Tn=reshape(Tn,[dims(1:n-2) 1 dims(n:l)]);
  end
end


    